from __future__ import absolute_import

from ..exceptions import ImproperlyConfigured
from .enriched_datetime import ArrowDateTime
from .enriched_datetime.enriched_datetime_type import EnrichedDateTimeType

arrow = None
try:
    import arrow
except ImportError:
    pass


class ArrowType(EnrichedDateTimeType):
    """
    ArrowType provides way of saving Arrow_ objects into database. It
    automatically changes Arrow_ objects to datetime objects on the way in and
    datetime objects back to Arrow_ objects on the way out (when querying
    database). ArrowType needs Arrow_ library installed.

    .. _Arrow: https://github.com/arrow-py/arrow

    ::

        from datetime import datetime
        from sqlalchemy_utils import ArrowType
        import arrow


        class Article(Base):
            __tablename__ = 'article'
            id = sa.Column(sa.Integer, primary_key=True)
            name = sa.Column(sa.Unicode(255))
            created_at = sa.Column(ArrowType)



        article = Article(created_at=arrow.utcnow())


    As you may expect all the arrow goodies come available:

    ::


        article.created_at = article.created_at.replace(hours=-1)

        article.created_at.humanize()
        # 'an hour ago'

    """
    def __init__(self, *args, **kwargs):
        if not arrow:
            raise ImproperlyConfigured(
                "'arrow' package is required to use 'ArrowType'"
            )

        super(ArrowType, self).__init__(
            datetime_processor=ArrowDateTime,
            *args,
            **kwargs
        )



import pytest
import sqlalchemy as sa
from sqlalchemy.orm import dynamic_loader

from sqlalchemy_utils.observer import observes


@pytest.fixture
def Director(Base):
    class Director(Base):
        __tablename__ = 'director'
        id = sa.Column(sa.Integer, primary_key=True)
        name = sa.Column(sa.String)
        movies = dynamic_loader('Movie', back_populates='director')

    return Director


@pytest.fixture
def Movie(Base, Director):
    class Movie(Base):
        __tablename__ = 'movie'
        id = sa.Column(sa.Integer, primary_key=True)
        name = sa.Column(sa.String)
        director_id = sa.Column(sa.Integer, sa.ForeignKey(Director.id))
        director = sa.orm.relationship(Director, back_populates='movies')
        director_name = sa.Column(sa.String)

        @observes('director')
        def director_observer(self, director):
            self.director_name = director.name

    return Movie


@pytest.fixture
def init_models(Director, Movie):
    pass


@pytest.mark.usefixtures('postgresql_dsn')
class TestObservesForDynamicRelationship:
    def test_add_observed_object(self, session, Director, Movie):
        steven = Director(name='Steven Spielberg')
        session.add(steven)
        jaws = Movie(name='Jaws', director=steven)
        session.add(jaws)
        session.commit()
        assert jaws.director_name == 'Steven Spielberg'

    def test_add_observed_object_from_backref(self, session, Director, Movie):
        jaws = Movie(name='Jaws')
        steven = Director(name='Steven Spielberg', movies=[jaws])
        session.add(steven)
        session.add(jaws)
        session.commit()
        assert jaws.director_name == 'Steven Spielberg'
